' ------------------------------------------------------------------------------------------------
' Initialize Game
' ------------------------------------------------------------------------------------------------
Function InitGame()

	' --------------------------------------------------------------------------------------------
	' Start logfile
	' --------------------------------------------------------------------------------------------
	LogInit()	
	TIME.TotalStart(LOCALE.Get("LOG_INITIALIZESTART"))
		
	' --------------------------------------------------------------------------------------------
	' Load Localization
	' --------------------------------------------------------------------------------------------
	LOCALE.path = PATH_Locale
	LOCALE.language = GAME_LANGUAGE
	LOCALE.Load

	' --------------------------------------------------------------------------------------------
	' Set FLAGS
	' --------------------------------------------------------------------------------------------
	FLAG.Add("VSYNC", 0)										' VSYNC Flag
	FLAG.Add("DELAY", 0)										' Delay Flag
	FLAG.Add("BLUR", 1)											' Blur Flag
	FLAG.Add("WIREFRAME", 0)									' Wireframe Flag
	FLAG.Add("GUI", 1)											' GUI Flag
	FLAG.Add("FONT", 0)											' Font Detail Flag
	FLAG.Add("FPS", 1)											' FPS measurement
	FLAG.Add("TITLE", 1)										' Title Flag
	FLAG.Add("ORBITS", 1)										' Orbit Lines Flag
	FLAG.Add("ZONES", 0)										' Zones Flag
	FLAG.Add("GLOW", 1)											' Glow Flag
	FLAG.Add("PLANETS", 1)										' Planet ID Flag
	FLAG.Add("PATH", 0)											' Path Flag
	
	FLAG.Add("MILKYWAY", 1)										' Milkyway Object Flag
	FLAG.Add("MILKYWAY_PANORAMA", 1)							' Milkyway Panorama Strip Flag
	FLAG.Add("MILKYWAY_STARDETAIL", 1)							' Milkyway StarDetail Flag
	FLAG.Add("MILKYWAY_DETAIL", 1)								' Milkyway Detail Flag
	FLAG.Add("MILKYWAY_FIXSTARS", 1)							' Milkyway Fixed Stars Flag
	
	TIME.Start()
	
	' ------------------------------------------------------------------------------------------------
	' Create Main Window
	' ------------------------------------------------------------------------------------------------
	MAXGUI_WINDOW = CreateWindow(PROJECT_TITLE, SCREEN_CENTERX - (SCREEN_WIDTH / 2), SCREEN_CENTERY - (SCREEN_HEIGHT / 2), SCREEN_WIDTH, SCREEN_HEIGHT, Null, WINDOW_CLIENTCOORDS)
	SetGadgetColor(MAXGUI_WINDOW, 0, 0, 0)

	' ------------------------------------------------------------------------------------------------
	' Create Graphics Canvas in Main Window
	' ------------------------------------------------------------------------------------------------
	MAXGUI_CANVAS = CreateCanvas(0, 0, ClientWidth(MAXGUI_WINDOW), ClientHeight(MAXGUI_WINDOW), MAXGUI_WINDOW)
	SetGadgetLayout MAXGUI_CANVAS, 1, 1, 1, 1
	ActivateGadget(MAXGUI_CANVAS)

	' ------------------------------------------------------------------------------------------------
	' miniB3D Graphics Mode settings
	' ------------------------------------------------------------------------------------------------
	TGlobal.width = ClientWidth(MAXGUI_WINDOW)
	TGlobal.Height = ClientHeight(MAXGUI_WINDOW)
	TGlobal.depth = SCREEN_DEPTH
	TGlobal.mode = SCREEN_MODE
	TGlobal.rate = TIMER_Safe

	' ------------------------------------------------------------------------------------------------
	' Init Canvas Graphics
	' ------------------------------------------------------------------------------------------------
	SetGraphics CanvasGraphics(MAXGUI_CANVAS)
	TGlobal.GraphicsInit()
	UpdateScaler()
		
	TIME.Stop(LOCALE.Get("LOG_CREATECANVAS"))
		
	' --------------------------------------------------------------------------------------------
	' Create Camera
	' --------------------------------------------------------------------------------------------
		
	TIME.Start()
	
	' Main Camera
	CAMERA:TCam = New TCam
	CAMERA.parent = PIVOT_CAMERA
	CAMERA.Add()
	CAMERA.RangeMin = 1
	CAMERA.RangeMax = 10000
	CAMERA.speed = 1.0 * 0.1
	CAMERA.X = 0.0
	CAMERA.Y = 0.0
	CAMERA.Z = 10.0
	CAMERA.TurnSpeed = 100.0
	CAMERA.RollSpeed = 100.0
	CAMERA.MX = SCREEN_WIDTH / 2
	CAMERA.MY = SCREEN_HEIGHT / 2
	CAMERA.UpdatePosition()
	CAMERA.Update()
					
	TIME.Stop(LOCALE.Get("LOG_CREATECAMERA"))

	' --------------------------------------------------------------------------------------------
	' Load MEDIA Files
	' --------------------------------------------------------------------------------------------
	LoadMedia()

	' --------------------------------------------------------------------------------------------
	' Create Objects
	' --------------------------------------------------------------------------------------------
	
	' --------------------------------------------------------------------------------------------
	' Milkyway
	' --------------------------------------------------------------------------------------------
	TIME.Start
	
	CreateMilkyway()
	Cls
	RenderWorld
	Cls
	Flip False
	
	TIME.Stop(LOCALE.Get("LOG_CREATEMILKYWAY"))
						
	' --------------------------------------------------------------------------------------------
	' Mouse
	' --------------------------------------------------------------------------------------------
	CenterMouse()
	HideMouse()
	
	' --------------------------------------------------------------------------------------------
	' Resolution
	' --------------------------------------------------------------------------------------------
	SetResolution(DesktopWidth(), DesktopHeight())
	'SetResolution(2048, 1024)
	'SetResolution(1920, 1080)
	
	If TSound(MEDIA.GetSound("AMBIENT")) Then CHANNEL_AMBIENT = CueSound(MEDIA.GetSound("AMBIENT")) Else StopChannel CHANNEL_AMBIENT
	SetChannelVolume(CHANNEL_AMBIENT, 0.25)
	
	If TSound(MEDIA.GetSound("ENGINEA")) Then CHANNEL_ENGINE1 = CueSound(MEDIA.GetSound("ENGINEA")) Else StopChannel CHANNEL_ENGINE1
	SetChannelVolume(CHANNEL_ENGINE1, 0.0)

	If TSound(MEDIA.GetSound("ENGINEB")) Then CHANNEL_ENGINE2 = CueSound(MEDIA.GetSound("ENGINEB")) Else StopChannel CHANNEL_ENGINE2
	SetChannelVolume(CHANNEL_ENGINE2, 0.0)

	If TSound(MEDIA.GetSound("RADIATION")) Then CHANNEL_RADIATION = CueSound(MEDIA.GetSound("RADIATION")) Else StopChannel CHANNEL_RADIATION
	If CHANNEL_RADIATION Then
	
		StopChannel CHANNEL_RADIATION
		SetChannelVolume(CHANNEL_RADIATION, 0.0)
		SetChannelRate(CHANNEL_RADIATION, 0.0)
		
	EndIf
	
	ResumeChannel CHANNEL_AMBIENT
	ResumeChannel CHANNEL_ENGINE1
	ResumeChannel CHANNEL_ENGINE2
	
	GRADIENT = New TGradient
		
	TIME.TotalStop(LOCALE.Get("LOG_INITIALIZESTOP"))

End Function



' ------------------------------------------------------------------------------------------------
' Initialize Milkyway background
' ------------------------------------------------------------------------------------------------
Function CreateMilkyway()
	
	MILKYWAY.StarSphereTexture = MEDIA.GetTexture("MILKYWAY_STAR")
	MILKYWAY.PanoramaTexture = MEDIA.GetTexture("MILKYWAY_STRIP")
	MILKYWAY.DetailTexture = MEDIA.GetTexture("MILKYWAY_DETAIL")
	MILKYWAY.StarsTexture = MEDIA.GetTexture("MILKYWAY_STARS")
	MILKYWAY.STARSCALE = 12 * Scaler
	
	MILKYWAY.PIVOT = PIVOT_MASTER
	MILKYWAY.Seed = 41'Rand(0, 2 ^ 16)
	MILKYWAY.Init
	
	RotateEntity MILKYWAY.PIVOT, 0, 205, 0

End Function